let origFetch;
let origXMLHttpRequest;
/**
 * NetworkInterceptor class to intercept network requests and responses
 */
export class NetworkInterceptor {
    addFetchInterceptor(onRequest, onResponse) {
        if (!window.fetch) {
            return;
        }
        origFetch = window.fetch;
        window.fetch = async (...args) => {
            try {
                onRequest(args);
            }
            catch (err) {
                console.log('Error handling request: ', err);
            }
            const response = await origFetch(...args);
            try {
                onResponse(response.clone());
            }
            catch (err) {
                console.log('Error handling response: ', err);
            }
            return response;
        };
    }
    addXhrInterceptor(onRequest, onResponse) {
        if (!window.XMLHttpRequest) {
            return;
        }
        const OrigXMLHttpRequest = window.XMLHttpRequest;
        class InterceptedXMLHttpRequest extends OrigXMLHttpRequest {
            constructor() {
                super();
                this.addEventListener('readystatechange', () => {
                    // Handle request
                    if (this.readyState === this.HEADERS_RECEIVED) {
                        try {
                            onRequest([
                                this._reqURL,
                                {
                                    body: this._reqBody,
                                    headers: this._reqHeaders,
                                    method: this._reqMethod,
                                    responseType: this.responseType,
                                },
                            ]);
                        }
                        catch (err) {
                            console.log('Error handling request', err);
                        }
                    }
                    // Handle response
                    if (this.readyState === this.DONE) {
                        try {
                            const headers = new Headers();
                            const allResponseHeaders = this.getAllResponseHeaders();
                            allResponseHeaders
                                .trim()
                                .split(/[\r\n]+/)
                                .forEach((line) => {
                                const parts = line.split(': ');
                                const header = parts.shift();
                                const value = parts.join(': ');
                                if (header) {
                                    headers.append(header, value);
                                }
                            });
                            onResponse({
                                status: this.status,
                                responseType: this.responseType,
                                statusText: this.statusText,
                                url: this.responseURL,
                                headers,
                                body: this.response,
                            });
                        }
                        catch (err) {
                            console.log('Error handling response', err);
                        }
                    }
                });
            }
            // @ts-ignore
            open(...args) {
                const [method, url] = args;
                try {
                    this._reqURL = url.toString();
                    this._reqMethod = method;
                }
                catch (err) {
                    console.log('Error handling request (open)', err);
                }
                super.open(...args);
            }
            send(body) {
                this._reqBody = body;
                super.send(body);
            }
            setRequestHeader(...args) {
                try {
                    const [header, value] = args;
                    if (!this._reqHeaders) {
                        this._reqHeaders = new Headers();
                    }
                    this._reqHeaders.append(header, value);
                }
                catch (err) {
                    console.log('Error handling request (setRequestHeader)', err);
                }
                super.setRequestHeader(...args);
            }
        }
        ;
        window.XMLHttpRequest = InterceptedXMLHttpRequest;
    }
    cleanup() {
        origXMLHttpRequest && (window.XMLHttpRequest = origXMLHttpRequest);
        origFetch && (window.fetch = origFetch);
    }
}
//# sourceMappingURL=network-interceptor.js.map